from functools import wraps
from .client import get_global_langfuse, get_global_callback_handler

def observe(**langfuse_kwargs):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            langfuse = get_global_langfuse()
            if langfuse is None:
                print("Warning: Langshark client is not initialized. Skipping observation.")
                return func(*args, **kwargs)
            
            span = langfuse.span(**langfuse_kwargs)
            
            try:
                result = func(*args, **kwargs)
                span.end()
                return result
            except Exception as e:
                span.end(error=str(e))
                raise

        return wrapper
    return decorator


# import uuid
# from langfuse import Langfuse
# from langfuse.callback import CallbackHandler

# DEFAULT_HOST = "https://langshark.fmops.kr"
# global_langfuse_client = None

# class LangSharkClient:
#     def __init__(self, secret_key, public_key, username, tracename, host=DEFAULT_HOST):
#         self.secret_key = secret_key
#         self.public_key = public_key
#         self.host = host
#         self.username = username
#         self.session_uuid = str(uuid.uuid4())
#         self.session_id = f"{username}_{self.session_uuid}"
#         self.trace_name = f"{username}_{tracename}"

#         self.langfuse = Langfuse(
#             secret_key=secret_key,
#             public_key=public_key,
#             host=host,
#         )

#         self.callback_handler = CallbackHandler(
#             secret_key=secret_key,
#             public_key=public_key,
#             host=host,
#             trace_name=self.trace_name,
#             session_id=self.session_id,
#             user_id=username
#         )

#     def auth_check(self):
#         return self.langfuse.auth_check() and self.callback_handler.langfuse.auth_check()

#     def get_session_id(self):
#         return self.session_id

# def getLangShark(secret_key, public_key, username, tracename, host=DEFAULT_HOST):
#     global global_langfuse_client
#     client = LangSharkClient(secret_key, public_key, username, tracename, host)
#     global_langfuse_client = client.langfuse
#     return client.langfuse, client.callback_handler

# def get_global_client():
#     global global_langfuse_client
#     if global_langfuse_client is None:
#         raise ValueError("Langshark client not initialized. Call getLangShark first.")
#     return global_langfuse_client

from langfuse import Langfuse
from langfuse.callback import CallbackHandler

_global_langfuse_client = None
_global_callback_handler = None

def getLangShark(secret_key, public_key, username, tracename):
    global _global_langfuse_client, _global_callback_handler
    
    langfuse = Langfuse(secret_key=secret_key, public_key=public_key)
    callback_handler = CallbackHandler(
        secret_key=secret_key,
        public_key=public_key,
        host="https://langshark.fmops.kr",
        trace_name=tracename,
        user_id=username
    )
    
    _global_langfuse_client = langfuse
    _global_callback_handler = callback_handler
    
    return langfuse, callback_handler

def get_global_langfuse():
    global _global_langfuse_client
    return _global_langfuse_client

def get_global_callback_handler():
    global _global_callback_handler
    return _global_callback_handler


from .client import getLangShark, get_global_langfuse, get_global_callback_handler
from .decorators import observe

__all__ = [
    "getLangShark",
    "get_global_langfuse",
    "get_global_callback_handler",
    "observe"
]



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="langshark",
    version="0.1.461",
    author="Your Name",
    author_email="your.email@example.com",
    description="A wrapper for Langfuse with enhanced functionality",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/langshark",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    python_requires=">=3.11",
    install_requires=[
        "langfuse","langchain"
    ],
)